@@ -4,7 +4,6 @@ from pycuda.compiler import SourceModule
 import pycuda.gpuarray as gpuarray
 import numpy as np
 import time
-from progress.bar import Bar
 
 # Start timer from 0
 def tic():
@@ -39,16 +38,24 @@ class Sequential:
         self.numLayers += 1
 
     # Forward pass
-    def propagate(self, inputValues):
-        self.layer[0].x = inputValues
-        for l in self.layer:
-            l.propagate()
+    def propagate(self,iteration):
+        for l in range(self.numLayers):
+            if (l==0):
+                self.layer[l].propagate(iteration=iteration)
+            else:
+                self.layer[l].propagate()
             
     # Backward pass
-    def backpropagate(self,label):
-        self.layer[self.numLayers-1].backpropagate(label)
-        for l in range(self.numLayers-2,-1,-1):
-            self.layer[l].backpropagate()
+    def backpropagate(self,iteration,label):
+        if (self.numLayers==1):
+            self.layer[self.numLayers-1].backpropagate(iteration=iteration,label=label)    
+        else:
+            self.layer[self.numLayers-1].backpropagate(label=label)
+            for l in range(self.numLayers-2,-1,-1):
+                if (l==0):
+                    self.layer[l].backpropagate(iteration=iteration)
+                else:
+                    self.layer[l].backpropagate()
 
     # Decision
     def inference(self,label,hits_d):
@@ -78,26 +85,21 @@ class Sequential:
 
     # Import dataset
     def importDataset(self,trainData,trainLabels,testData,testLabels):
-        numTrain = len(trainData)
-        numTest = len(testData)
-        self.trainData       = []
-        self.trainLabels     = []
-        self.testData        = []
-        self.testLabels      = []
-        print()
-        bar = Bar('Importing dataset', max=int((numTrain+numTest)/1000), suffix='%(percent)d%%')
-        for i in range(numTrain):
-            self.trainData.append(gpuarray.to_gpu(trainData[i]))
-            if (i%1000 == 1):
-                bar.next()
-        self.trainLabels = np.int32(trainLabels)
-        for i in range(numTest):
-            self.testData.append(gpuarray.to_gpu(testData[i]))
-            if (i%1000 == 1):
-                bar.next()
-        bar.finish()
-        print()
-        self.testLabels = np.int32(testLabels)
+        """Send the dataset to the GPU.
+
+        trainData : dtype=np.float64()
+            Data to fit the network to. Currently supports 2D.
+        trainLabels : dtype=np.float64()
+            Labels corresponding to the inputs in trainData.
+        trainData : dtype=np.float64()
+            Data to validate the network with. Currently supports 2D.
+        trainData : dtype=np.float64()
+            Labels corresponding to the inputs in testData.
+        """
+        self.trainData      = gpuarray.to_gpu(trainData)
+        self.trainLabels    = trainLabels
+        self.testData       = gpuarray.to_gpu(testData)
+        self.testLabels     = testLabels
 
     # Test model
     def validate(self):
@@ -105,7 +107,7 @@ class Sequential:
         testHits = gpuarray.zeros(3,dtype=np.float64)
         self.layer[self.numLayers-1].resetHits()
         for i in range(numTests):
-            self.propagate(self.testData[i])
+            self.propagate(i)
             self.inference(self.testLabels[i], testHits)
         accuracy = testHits.get()[0]/numTests
         loss = 1-accuracy
@@ -128,9 +130,8 @@ class Sequential:
             self.history.alpha.append(l.alpha)
             self.history.beta.append(l.beta)
             self.history.sigma_i.append(l.sigma_i)
-        testData = validation_data[0]
-        testLabels = validation_data[1]
-        self.importDataset(trainData,trainLabels,testData,testLabels)
+        self.importDataset(trainData,trainLabels,validation_data[0],validation_data[1])
+        self.layer[0].x = self.trainData
         self.numTrain = len(trainData)
         self.numConverged = 0
         self.peakAccuracy = 0
@@ -143,9 +144,11 @@ class Sequential:
             self.layer[self.numLayers-1].resetHits()
             for self.i in range(self.numTrain):
                 if (self.i%(int(self.numTrain/tests_per_epoch))==0):
+                    self.layer[0].x = self.testData
                     self.validate()
-                self.propagate(self.trainData[self.i])
-                self.backpropagate(self.trainLabels[self.i])
+                    self.layer[0].x = self.trainData
+                self.propagate(self.i)
+                self.backpropagate(self.i,self.trainLabels[self.i])
                 self.inference(self.trainLabels[self.i],trainHits)
             accuracy = trainHits.get()[0]/self.numTrain
             loss = 1 - accuracy
